package org.example;

import org.graphstream.algorithm.Dijkstra;
import org.graphstream.algorithm.generator.Generator;
import org.graphstream.algorithm.generator.RandomGenerator;
import org.graphstream.graph.*;
import org.graphstream.graph.implementations.*;
import org.graphstream.stream.GraphParseException;

import java.io.IOException;
import java.util.*;

public class Main
{
    public static Graph Generateur(int taille, int degre){
        Graph graph = new SingleGraph("Random");
        graph.setAttribute("ui.stylesheet", "url('src/resources/style.css')");
        //création d'un graph aléatoirement
        Generator gen = new RandomGenerator(degre);
        gen.addSink(graph);
        gen.begin();
        for(int i=0; i<taille; i++)
            gen.nextEvents();
        gen.end();
        // ajout des poids aléatoires générer par la méthode  Random() sur nos arêtes
        // attributions d'une valeur numérique à chaque attribut « poid » des arêtes.
        for(Edge e : graph.getEachEdge()){
            Random r = new Random();
            e.setAttribute("poid", r.nextInt(100));
            e.setAttribute("label", (int)e.getAttribute("poid"));
        }
        // attributions d'une valeur numérique à chaque attribut à chaque attribut « label » des noeuds.
        for(Node n : graph){
            n.setAttribute("label", n.getId());
        }
        return graph;
    }

    public static Node extractMin(List<Node> list, Node n){
        // return le noeud a priorite minimum de notre noeud n de la liste list
        // min => distance minimum
        int min = Integer.MAX_VALUE ;
        // minNode => noeud a priorite minimum
        Node minNode = list.get(0);
        for (int i =0 ;  i<list.size(); i ++) {
            if (((list.get(i).getEdgeBetween(n.getId())) != null) && (min > ((int) (list.get(i)).getAttribute("distance") + (int) list.get(i).getEdgeBetween(n).getAttribute("poid")))) {
                min = ((int) (list.get(i)).getAttribute("distance") + (int) list.get(i).getEdgeBetween(n).getAttribute("poid"));
                minNode = list.get(i);
            }
        }

        return minNode;
    }

    public static void Dijkstra(Graph g, Node s){
        Integer infini = Integer.MAX_VALUE;
        // initialiser tout les nœuds avec une distance infinie, et leurs attributs parents à 0.
        for(Node n: g){
            n.setAttribute("distance", infini);
            n.setAttribute("parents", 0);
        }
        // On attribue 0 à la distance de la source
        s.setAttribute("distance",0);
        // creation d'une liste qu'on va parcourir par la suite
        List<Node> list1 = new ArrayList<Node>();
        // ajout du noeud source a notre liste
        list1.add(s);
        while(!list1.isEmpty()){
            Node noeudCourant = extractMin(list1, list1.get(0));

            for (Iterator<Node> it = noeudCourant.getNeighborNodeIterator(); it.hasNext(); ) {
                //  un pointeur qui pointe sur le noeud voisin du noeud courant
                Node voisin = it.next();
                // si y a une arrete dans le bon sens entre le noeud courant et le voisin dans le cas d'un graph orienté
                if (((noeudCourant.getEdgeBetween(voisin.getId())) != null)  && (int) voisin.getAttribute("distance") > ((int) noeudCourant.getAttribute("distance") + (int) noeudCourant.getEdgeBetween(voisin).getAttribute("poid"))) {
                    // on mets à jour l'attribut distance et parents du noeud
                    voisin.setAttribute("distance", (int) noeudCourant.getAttribute("distance") + (int) noeudCourant.getEdgeBetween(voisin).getAttribute("poid"));
                    voisin.setAttribute("parents", noeudCourant);
                    //on ajout le voisin à notre liste
                    list1.add(voisin);
                    noeudCourant.getEdgeBetween(voisin).setAttribute("ui.style","fill-color:red;");
                    voisin.setAttribute("ui.style", "fill-color: blue;");
                }
                else continue;
            }
            //on suprrime le noeud courant
            list1.remove(noeudCourant);
        }
        // on affiche le plus cours chemin entre la source et tous le reste du noeud du graph
        for(Node n  : g){
            System.out.println("Plus court Chemin etre la source [" +s+ "] et le Noeud ["+ n + "] est de distance:" +n.getAttribute("distance"));
        }
    }

    public static Graph premierExemple() throws IOException, GraphParseException {
        Graph graph = new SingleGraph("premier exemple");
        // on crees un graph d'exemple avec le fichier dgs
        graph.setAttribute("ui.stylesheet", "url('src/resources/style.css')");
        graph.read("src/resources/example.dgs");
        // on ajout les poid sur les arretes
        for (Edge e : graph.getEachEdge()) {
            e.setAttribute("poid", (int) e.getAttribute("ui.label"));
        }
        return graph;
    }

    public static void tempExecutionPlusieurGraphIdentiques(int debut, int nbIterations, int degre){
        //  des graphes construits de la même manière
        // creation de deux tableaux pour nos resultats
        long[] resultatsCustom = new long[nbIterations-debut];
        long[] resultatsGraphStream = new long[nbIterations-debut];
        int index = 0;
        for (int i =debut; i<nbIterations; i++) {
            Graph graph = Generateur(i, degre);
            Node n = graph.getNode(0);
            // stockage du temps d'execution dans nos tableaux
            resultatsCustom[index] = tempExecution(graph, n);;
            resultatsGraphStream[index] = tempExecutionGraphStream(graph, n);
            index++;
        }
        System.out.println("Temps d'execution de notre algo :");
        System.out.println("\t\tMinimal: " + Arrays.stream(resultatsCustom).min().getAsLong() + "ms.");
        System.out.println("\t\tMaximal: " + Arrays.stream(resultatsCustom).max().getAsLong() + "ms.");
        System.out.println("\t\tMoyenne: " + Arrays.stream(resultatsCustom).average().getAsDouble() + "ms.");
        System.out.println("Temps d'execution de l'algo du GraphStream :");
        System.out.println("\t\tMinimal: " + Arrays.stream(resultatsGraphStream).min().getAsLong() + "ms.");
        System.out.println("\t\tMaximal: " + Arrays.stream(resultatsGraphStream).max().getAsLong() + "ms.");
        System.out.println("\t\tMoyenne: " + Arrays.stream(resultatsGraphStream).average().getAsDouble() + "ms.");
    }

    public static void tempExecutionPlusieurGraphDifferent(int nbIterations, int min, int max, int degre){
        // des graphes de différentes tailles
        // creation de deux tableaux pour nos resultats
        long[] resultatsCustom = new long[nbIterations];
        long[] resultatsGraphStream = new long[nbIterations];

        for (int i =0; i<nbIterations; i++) {
            Random r = new Random();
            // creation d'un graph aleatoire de taille aleatoire entre[min,max]
            Graph graph = Generateur(r.nextInt(max+1-min)+min, degre);
            Node n = graph.getNode(0);
            // stockage du temps d'execution dans nos tableaux
            resultatsCustom[i] = tempExecution(graph, n);;
            resultatsGraphStream[i] = tempExecutionGraphStream(graph, n);
        }

        System.out.println("Temps d'execution de notre algo :");
        System.out.println("\t\tMinimal: " + Arrays.stream(resultatsCustom).min().getAsLong() + "ms.");
        System.out.println("\t\tMaximal: " + Arrays.stream(resultatsCustom).max().getAsLong() + "ms.");
        System.out.println("\t\tMoyenne: " + Arrays.stream(resultatsCustom).average().getAsDouble() + "ms.");
        System.out.println("Temps d'execution de l'algo du GraphStream :");
        System.out.println("\t\tMinimal: " + Arrays.stream(resultatsGraphStream).min().getAsLong() + "ms.");
        System.out.println("\t\tMaximal: " + Arrays.stream(resultatsGraphStream).max().getAsLong() + "ms.");
        System.out.println("\t\tMoyenne: " + Arrays.stream(resultatsGraphStream).average().getAsDouble() + "ms.");
    }

    public static long tempExecution(Graph g, Node n){
        long startTime = System.currentTimeMillis();
        Dijkstra(g,n);
        long endTime = System.currentTimeMillis();
        // le temps d'execution est la differance du temps du fin et le temps du debut
        long duration = (endTime - startTime);
        return duration;
    }

    public static long tempExecutionGraphStream(Graph g, Node n){
        Dijkstra dijkstra = new Dijkstra();
        long startTime = System.currentTimeMillis();
        dijkstra.init(g);
        dijkstra.setSource(n);
        dijkstra.compute();
        long endTime = System.currentTimeMillis();
        // le temps d'execution est la differance du temps du fin et le temps du debut
        long duration = (endTime - startTime);
        return duration;
    }


    public static void main(String args[]) throws IOException, GraphParseException {
        //graphe aleatoire de taille 30 et degre 5
        // premiere execution
        /*
        Graph graph2 = Generateur(30,5);
        graph2.display();

        */


        //execution exemple 1
        // deuxieme execution
        // creation d'un graph a partir du fichier DGS
        /*
        Graph graph = premierExemple();
        graph.display(false);
        Node s = graph.getNode("A");
        Dijkstra(graph,s);
        */

        //execution exemple 2
        // 3eme execution
        // creation d'un graphe aleatoire
        /*
        Graph graph2 = Generateur(100,20);
        graph2.display();
        Node s2 = graph2.getNode(0);
        // application de notre algo et celui du GraphStream
        Dijkstra(graph2,s2);
        */

        // 4eme execution
        /*
        Graph graph2 = Generateur(100,20);
        graph2.display();
        // Les longueurs d'arête sont stockées dans un attribut appelé "longueur"
        // La longueur d'un chemin est la somme des longueurs de ses arêtes
        Dijkstra dijkstra = new Dijkstra(Dijkstra.Element.EDGE, null, "poid");
        dijkstra.init(graph2);
        // Calculer les chemins les plus courts en graph2 de graph2.getNode(0) à tous les nœuds
        dijkstra.setSource(graph2.getNode(0));
        dijkstra.compute();
        // print les longueurs de tous les chemins les plus courts
        for (Node node : graph2)
            System.out.printf("%s->%s:%10.2f%n", dijkstra.getSource(), node,
                    dijkstra.getPathLength(node));
        // Colorier en rouge toutes les arêtes du graph des chemins les plus courts
        for (Edge edge : dijkstra.getTreeEdges())
            edge.setAttribute("ui.style", "fill-color: red;");
        */

        // 5 eme execution
        Graph graph2 = Generateur(100,20);
        Node s2 = graph2.getNode(0);
        // calcul temp d'execution de notre algo et celui du GraphStream
        long temp1 = tempExecution(graph2,s2);
        long temp2 = tempExecutionGraphStream(graph2,s2);
        System.out.println("\t\tTemps d'execution de notre algo : " + temp1 + "ms.");
        System.out.println("\t\tTemps d'execution de l'algo du GraphStream : " + temp2 + "ms.");

        //6eme execution
        // temps d'execution pour plusieurs graph  identiques
        //tempExecutionPlusieurGraphIdentiques(1990,2000,20);
        //7eme execution
        // temps d'execution pour plusieurs graph  differents
        //tempExecutionPlusieurGraphDifferent(1, 4999,5000,20);

    }
}